import { Model } from 'mongoose';
import { Injectable } from '@nestjs/common';
import { InjectModel } from '@nestjs/mongoose';
import { Property } from './property.model';
import * as _ from 'lodash'
@Injectable()
export class PropertyService {

  constructor(@InjectModel('Property') private propertyModel: Model<Property>) {}

  async createOrUpdate(oldData, newData) {
    return await this.propertyModel.create(oldData, newData, { upsert: true, new: true, setDefaultsOnInsert: true });
  }

  async updateProperty(oldData, newData) : Promise<any>{
    return await this.propertyModel.findOneAndUpdate({reference_number: oldData?.reference_number}, newData)
  }


  async updateManyProperties(oldData, newData) : Promise<any>{
    return await this.propertyModel.updateMany(oldData, newData)
  }

  async findOneByProperty(reference_number) : Promise<any>{
    return await this.propertyModel.findOne({reference_number: reference_number})
  }

  async getAllProperties() : Promise<any>{
    return this.propertyModel.find()
  }

  async getAllPropertiesFiltered(params?) : Promise<any>{

    let limit = Number(_.get(params, 'limit', 20))
    let page = Number(_.get(params, 'page', 0))
    let price_lt = Number(_.get(params, 'price_lt', 9999999999))
    let price_gt = Number(_.get(params, 'price_gt', 1))
    let bathroom = Number(_.get(params, 'bathroom', 1))
    let bedroom = Number(_.get(params, 'bedroom', 1))


    let search = _.get(params, 'search', [])
    let type = _.get(params, 'property_type', [])

    let orData = _.size(_.compact(search))>0 ? { $or :[
        {title_en: {$in: search}}, 
        {community: {$in: search}}, 
        {sub_community: {$in: search}},
        {city: {$in: search}},
        {reference_number: {$in: search}},
        {developer: {$in: search}},
      ]} : {}

    let andData: any = [
      { price : { $gte: price_gt } },
      { price : { $lte: price_lt } },
      { bedroom : { $gte: bedroom } },
      { bathroom : { $gte: bathroom } },
    ]

    if(_.size(type)>0){
      andData.push({ property_type : { $in: type } })
    }
      
    
    return await this.propertyModel.find({ 
                    ...orData,
                    $and : andData
                  })
                  .skip(limit*page).limit(limit)
      
  }



}